@@ -22,28 +22,36 @@
 #  SOFTWARE.
 #  ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~
 
+import logging
+import warnings
 from dataclasses import dataclass
 from numbers import Number
 from typing import Any
 from typing import Dict
+from typing import final
 from typing import Sequence
 from typing import Tuple
 from typing import Union
-from typing import final
 
 import torch
 
-from disent.frameworks._ae_mixin import _AeAndVaeMixin
+from disent.frameworks import DisentFramework
+from disent.frameworks.helper.reconstructions import make_reconstruction_loss
+from disent.frameworks.helper.reconstructions import ReconLossHandler
 from disent.frameworks.helper.util import detach_all
 from disent.model import AutoEncoder
 from disent.util.iters import map_all
 
+
+log = logging.getLogger(__name__)
+
+
 # ========================================================================= #
 # framework_vae                                                             #
 # ========================================================================= #
 
 
-class Ae(_AeAndVaeMixin):
+class Ae(DisentFramework):
     """
     Basic Auto Encoder
     ------------------
@@ -65,23 +73,56 @@ class Ae(_AeAndVaeMixin):
         * `compute_ave_recon_loss`
     """
 
-    # override
     REQUIRED_Z_MULTIPLIER = 1
     REQUIRED_OBS = 1
 
     @dataclass
-    class cfg(_AeAndVaeMixin.cfg):
-        pass
+    class cfg(DisentFramework.cfg):
+        recon_loss: str = 'mse'
+        # multiple reduction modes exist for the various loss components.
+        # - 'sum': sum over the entire batch
+        # - 'mean': mean over the entire batch
+        # - 'mean_sum': sum each observation, returning the mean sum over the batch
+        loss_reduction: str = 'mean'
+        # disable various components
+        disable_decoder: bool = False
+        disable_rec_loss: bool = False
+        disable_aug_loss: bool = False
 
-    def __init__(self, model: AutoEncoder, cfg: cfg = None, batch_augment=None):
-        super().__init__(cfg=cfg, batch_augment=batch_augment)
-        # initialise the auto-encoder mixin (recon handler, model, enc, dec, etc.)
-        self._init_ae_mixin(model=model)
+    def __init__(self, make_optimizer_fn, make_model_fn, batch_augment=None, cfg: cfg = None):
+        super().__init__(make_optimizer_fn, batch_augment=batch_augment, cfg=cfg)
+        # vae model
+        assert callable(make_model_fn)
+        self._model: AutoEncoder = make_model_fn()  # TODO: move into property
+        # check the model
+        assert isinstance(self._model, AutoEncoder)
+        assert self._model.z_multiplier == self.REQUIRED_Z_MULTIPLIER, f'model z_multiplier is {repr(self._model.z_multiplier)} but {self.__class__.__name__} requires that it is: {repr(self.REQUIRED_Z_MULTIPLIER)}'
+        # recon loss & activation fn
+        self.__recon_handler: ReconLossHandler = make_reconstruction_loss(self.cfg.recon_loss, reduction=self.cfg.loss_reduction)
+
+    @final
+    @property
+    def recon_handler(self) -> ReconLossHandler:
+        return self.__recon_handler
 
     # --------------------------------------------------------------------- #
     # AE Training Step -- Overridable                                       #
     # --------------------------------------------------------------------- #
 
+    @final
+    def _get_xs_and_targs(self, batch: Dict[str, Tuple[torch.Tensor, ...]], batch_idx) -> Tuple[Tuple[torch.Tensor, ...], Tuple[torch.Tensor, ...]]:
+        xs_targ = batch['x_targ']
+        if 'x' not in batch:
+            warnings.warn('dataset does not have input: x -> x_targ using target as input: x_targ -> x_targ')
+            xs = xs_targ
+        else:
+            xs = batch['x']
+        # check that we have the correct number of inputs
+        if (len(xs) != self.REQUIRED_OBS) or (len(xs_targ) != self.REQUIRED_OBS):
+            log.warning(f'batch len(xs)={len(xs)} and len(xs_targ)={len(xs_targ)} observation count mismatch, requires: {self.REQUIRED_OBS}')
+        # done
+        return xs, xs_targ
+
     @final
     def do_training_step(self, batch, batch_idx):
         xs, xs_targ = self._get_xs_and_targs(batch, batch_idx)
@@ -90,10 +131,10 @@ class Ae(_AeAndVaeMixin):
         # -~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~- #
         # latent variables
         zs = map_all(self.encode, xs)
-        # [HOOK] intercept latent variables
+        # intercept latent variables
         zs, logs_intercept_zs = self.hook_ae_intercept_zs(zs)
         # reconstruct without the final activation
-        xs_partial_recon = map_all(self.decode_partial, detach_all(zs, if_=self.cfg.detach_decoder))
+        xs_partial_recon = map_all(self.decode_partial, detach_all(zs, if_=self.cfg.disable_decoder))
         # -~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~- #
 
         # LOSS
@@ -101,37 +142,21 @@ class Ae(_AeAndVaeMixin):
         # compute all the recon losses
         recon_loss, logs_recon = self.compute_ave_recon_loss(xs_partial_recon, xs_targ)
         # [HOOK] augment loss
-        aug_loss, logs_aug = self.hook_ae_compute_ave_aug_loss(
-            zs=zs, xs_partial_recon=xs_partial_recon, xs_targ=xs_targ
-        )
+        aug_loss, logs_aug = self.hook_ae_compute_ave_aug_loss(zs=zs, xs_partial_recon=xs_partial_recon, xs_targ=xs_targ)
         # compute combined loss
         loss = 0
-        if not self.cfg.disable_rec_loss:
-            loss += recon_loss
-        if not self.cfg.disable_aug_loss:
-            loss += aug_loss
+        if not self.cfg.disable_rec_loss: loss += recon_loss
+        if not self.cfg.disable_aug_loss: loss += aug_loss
         # -~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~- #
 
-        # log general
-        self.log_dict(
-            {
-                **logs_intercept_zs,
-                **logs_recon,
-                **logs_aug,
-            }
-        )
-
-        # log progress bar
-        self.log_dict(
-            {
-                "recon_loss": float(recon_loss),
-                "aug_loss": float(aug_loss),
-            },
-            prog_bar=True,
-        )
-
         # return values
-        return loss
+        return loss, {
+            **logs_intercept_zs,
+            **logs_recon,
+            **logs_aug,
+            'recon_loss': recon_loss,
+            'aug_loss': aug_loss,
+        }
 
     # --------------------------------------------------------------------- #
     # Overrideable                                                          #
@@ -140,18 +165,16 @@ class Ae(_AeAndVaeMixin):
     def hook_ae_intercept_zs(self, zs: Sequence[torch.Tensor]) -> Tuple[Sequence[torch.Tensor], Dict[str, Any]]:
         return zs, {}
 
-    def hook_ae_compute_ave_aug_loss(
-        self, zs: Sequence[torch.Tensor], xs_partial_recon: Sequence[torch.Tensor], xs_targ: Sequence[torch.Tensor]
-    ) -> Tuple[Union[torch.Tensor, Number], Dict[str, Any]]:
+    def hook_ae_compute_ave_aug_loss(self, zs: Sequence[torch.Tensor], xs_partial_recon: Sequence[torch.Tensor], xs_targ: Sequence[torch.Tensor]) -> Tuple[Union[torch.Tensor, Number], Dict[str, Any]]:
         return 0, {}
 
-    def compute_ave_recon_loss(
-        self, xs_partial_recon: Sequence[torch.Tensor], xs_targ: Sequence[torch.Tensor]
-    ) -> Tuple[Union[torch.Tensor, Number], Dict[str, Any]]:
+    def compute_ave_recon_loss(self, xs_partial_recon: Sequence[torch.Tensor], xs_targ: Sequence[torch.Tensor]) -> Tuple[Union[torch.Tensor, Number], Dict[str, Any]]:
         # compute reconstruction loss
         pixel_loss = self.recon_handler.compute_ave_loss_from_partial(xs_partial_recon, xs_targ)
         # return logs
-        return pixel_loss, {"pixel_loss": pixel_loss}
+        return pixel_loss, {
+            'pixel_loss': pixel_loss
+        }
 
     # --------------------------------------------------------------------- #
     # AE Model Utility Functions (Visualisation)                            #
