@@ -26,26 +26,31 @@ from dataclasses import dataclass
 from numbers import Number
 from typing import Any
 from typing import Dict
+from typing import final
+from typing import Optional
 from typing import Sequence
 from typing import Tuple
 from typing import Union
-from typing import final
 
 import torch
 from torch.distributions import Distribution
+from torch.distributions import Laplace
+from torch.distributions import Normal
 
-from disent.frameworks._ae_mixin import _AeAndVaeMixin
+from disent.frameworks.ae._unsupervised__ae import Ae
 from disent.frameworks.helper.latent_distributions import LatentDistsHandler
 from disent.frameworks.helper.latent_distributions import make_latent_distribution
 from disent.frameworks.helper.util import detach_all
+
 from disent.util.iters import map_all
 
+
 # ========================================================================= #
 # framework_vae                                                             #
 # ========================================================================= #
 
 
-class Vae(_AeAndVaeMixin):
+class Vae(Ae):
     """
     Variational Auto Encoder
     https://arxiv.org/abs/1312.6114
@@ -83,27 +88,24 @@ class Vae(_AeAndVaeMixin):
           Vae(hook_compute_ave_aug_loss=TripletLoss(), required_obs=3)
     """
 
-    # overrides
+    # override required z from AE
     REQUIRED_Z_MULTIPLIER = 2
     REQUIRED_OBS = 1
 
     @dataclass
-    class cfg(_AeAndVaeMixin.cfg):
+    class cfg(Ae.cfg):
         # latent distribution settings
-        latent_distribution: str = "normal"
-        kl_loss_mode: str = "direct"
+        latent_distribution: str = 'normal'
+        kl_loss_mode: str = 'direct'
         # disable various components
         disable_reg_loss: bool = False
+        disable_posterior_scale: Optional[float] = None
 
-    def __init__(self, model: "AutoEncoder", cfg: cfg = None, batch_augment=None):
+    def __init__(self, make_optimizer_fn, make_model_fn, batch_augment=None, cfg: cfg = None):
         # required_z_multiplier
-        super().__init__(cfg=cfg, batch_augment=batch_augment)
-        # initialise the auto-encoder mixin (recon handler, model, enc, dec, etc.)
-        self._init_ae_mixin(model=model)
+        super().__init__(make_optimizer_fn, make_model_fn, batch_augment=batch_augment, cfg=cfg)
         # vae distribution
-        self.__latents_handler = make_latent_distribution(
-            self.cfg.latent_distribution, kl_mode=self.cfg.kl_loss_mode, reduction=self.cfg.loss_reduction
-        )
+        self.__latents_handler = make_latent_distribution(self.cfg.latent_distribution, kl_mode=self.cfg.kl_loss_mode, reduction=self.cfg.loss_reduction)
 
     @final
     @property
@@ -122,12 +124,14 @@ class Vae(_AeAndVaeMixin):
         # -~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~- #
         # latent distribution parameterizations
         ds_posterior, ds_prior = map_all(self.encode_dists, xs, collect_returned=True)
+        # [HOOK] disable learnt scale values
+        ds_posterior, ds_prior = self._hook_intercept_ds_disable_scale(ds_posterior, ds_prior)
         # [HOOK] intercept latent parameterizations
         ds_posterior, ds_prior, logs_intercept_ds = self.hook_intercept_ds(ds_posterior, ds_prior)
         # sample from dists
         zs_sampled = tuple(d.rsample() for d in ds_posterior)
         # reconstruct without the final activation
-        xs_partial_recon = map_all(self.decode_partial, detach_all(zs_sampled, if_=self.cfg.detach_decoder))
+        xs_partial_recon = map_all(self.decode_partial, detach_all(zs_sampled, if_=self.cfg.disable_decoder))
         # -~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~- #
 
         # LOSS
@@ -137,85 +141,70 @@ class Vae(_AeAndVaeMixin):
         # compute all the regularization losses
         reg_loss, logs_reg = self.compute_ave_reg_loss(ds_posterior, ds_prior, zs_sampled)
         # [HOOK] augment loss
-        aug_loss, logs_aug = self.hook_compute_ave_aug_loss(
-            ds_posterior=ds_posterior,
-            ds_prior=ds_prior,
-            zs_sampled=zs_sampled,
-            xs_partial_recon=xs_partial_recon,
-            xs_targ=xs_targ,
-        )
+        aug_loss, logs_aug = self.hook_compute_ave_aug_loss(ds_posterior=ds_posterior, ds_prior=ds_prior, zs_sampled=zs_sampled, xs_partial_recon=xs_partial_recon, xs_targ=xs_targ)
         # compute combined loss
         loss = 0
-        if not self.cfg.disable_rec_loss:
-            loss += recon_loss
-        if not self.cfg.disable_aug_loss:
-            loss += aug_loss
-        if not self.cfg.disable_reg_loss:
-            loss += reg_loss
+        if not self.cfg.disable_rec_loss: loss += recon_loss
+        if not self.cfg.disable_aug_loss: loss += aug_loss
+        if not self.cfg.disable_reg_loss: loss += reg_loss
         # -~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~- #
 
-        # log general
-        self.log_dict(
-            {
-                **logs_intercept_ds,
-                **logs_recon,
-                **logs_reg,
-                **logs_aug,
-            }
-        )
-
-        # log progress bar
-        self.log_dict(
-            {
-                "recon_loss": float(recon_loss),
-                "reg_loss": float(reg_loss),
-                "aug_loss": float(aug_loss),
-            },
-            prog_bar=True,
-        )
-
         # return values
-        return loss
+        return loss, {
+            **logs_intercept_ds,
+            **logs_recon,
+            **logs_reg,
+            **logs_aug,
+            'recon_loss': recon_loss,
+            'reg_loss': reg_loss,
+            'aug_loss': aug_loss,
+        }
+
+    # --------------------------------------------------------------------- #
+    # Delete AE Hooks                                                       #
+    # --------------------------------------------------------------------- #
+
+    @final
+    def hook_ae_intercept_zs(self, zs: Sequence[torch.Tensor]) -> Tuple[Sequence[torch.Tensor], Dict[str, Any]]:
+        raise RuntimeError('This function should never be used or overridden by VAE methods!')  # pragma: no cover
+
+    @final
+    def hook_ae_compute_ave_aug_loss(self, zs: Sequence[torch.Tensor], xs_partial_recon: Sequence[torch.Tensor], xs_targ: Sequence[torch.Tensor]) -> Tuple[Union[torch.Tensor, Number], Dict[str, Any]]:
+        raise RuntimeError('This function should never be used or overridden by VAE methods!')  # pragma: no cover
+
+    # --------------------------------------------------------------------- #
+    # Private Hooks                                                         #
+    # --------------------------------------------------------------------- #
+
+    def _hook_intercept_ds_disable_scale(self, ds_posterior: Sequence[Distribution], ds_prior: Sequence[Distribution]):
+        # disable posterior scales
+        if self.cfg.disable_posterior_scale is not None:
+            for d_posterior in ds_posterior:
+                assert isinstance(d_posterior, (Normal, Laplace))
+                d_posterior.scale = torch.full_like(d_posterior.scale, fill_value=self.cfg.disable_posterior_scale)
+        # return modified values
+        return ds_posterior, ds_prior
 
     # --------------------------------------------------------------------- #
     # Overrideable Hooks                                                    #
     # --------------------------------------------------------------------- #
 
-    def hook_intercept_ds(
-        self, ds_posterior: Sequence[Distribution], ds_prior: Sequence[Distribution]
-    ) -> Tuple[Sequence[Distribution], Sequence[Distribution], Dict[str, Any]]:
+    def hook_intercept_ds(self, ds_posterior: Sequence[Distribution], ds_prior: Sequence[Distribution]) -> Tuple[Sequence[Distribution], Sequence[Distribution], Dict[str, Any]]:
         return ds_posterior, ds_prior, {}
 
-    def hook_compute_ave_aug_loss(
-        self,
-        ds_posterior: Sequence[Distribution],
-        ds_prior: Sequence[Distribution],
-        zs_sampled: Sequence[torch.Tensor],
-        xs_partial_recon: Sequence[torch.Tensor],
-        xs_targ: Sequence[torch.Tensor],
-    ) -> Tuple[Union[torch.Tensor, Number], Dict[str, Any]]:
+    def hook_compute_ave_aug_loss(self, ds_posterior: Sequence[Distribution], ds_prior: Sequence[Distribution], zs_sampled: Sequence[torch.Tensor], xs_partial_recon: Sequence[torch.Tensor], xs_targ: Sequence[torch.Tensor]) -> Tuple[Union[torch.Tensor, Number], Dict[str, Any]]:
         return 0, {}
 
-    def compute_ave_recon_loss(
-        self, xs_partial_recon: Sequence[torch.Tensor], xs_targ: Sequence[torch.Tensor]
-    ) -> Tuple[Union[torch.Tensor, Number], Dict[str, Any]]:
-        # compute reconstruction loss
-        pixel_loss = self.recon_handler.compute_ave_loss_from_partial(xs_partial_recon, xs_targ)
-        # return logs
-        return pixel_loss, {"pixel_loss": pixel_loss}
-
-    def compute_ave_reg_loss(
-        self, ds_posterior: Sequence[Distribution], ds_prior: Sequence[Distribution], zs_sampled: Sequence[torch.Tensor]
-    ) -> Tuple[Union[torch.Tensor, Number], Dict[str, Any]]:
+    def compute_ave_reg_loss(self, ds_posterior: Sequence[Distribution], ds_prior: Sequence[Distribution], zs_sampled: Sequence[torch.Tensor]) -> Tuple[Union[torch.Tensor, Number], Dict[str, Any]]:
         # compute regularization loss (kl divergence)
         kl_loss = self.latents_handler.compute_ave_kl_loss(ds_posterior, ds_prior, zs_sampled)
         # return logs
         return kl_loss, {
-            "kl_loss": kl_loss,
+            'kl_loss': kl_loss,
         }
 
     # --------------------------------------------------------------------- #
-    # VAE Model Utility Functions (Visualisation)                           #
+    # VAE - Encoding - Overrides AE                                         #
     # --------------------------------------------------------------------- #
 
     @final
@@ -225,20 +214,6 @@ class Vae(_AeAndVaeMixin):
         z = self.latents_handler.encoding_to_representation(z_raw)
         return z
 
-    @final
-    def decode(self, z: torch.Tensor) -> torch.Tensor:
-        """Decode latent vector z into reconstruction x_recon (useful for visualisation)"""
-        return self.recon_handler.activate(self._model.decode(z))
-
-    @final
-    def forward(self, batch: torch.Tensor) -> torch.Tensor:
-        """Feed through the full deterministic model (useful for visualisation)"""
-        return self.decode(self.encode(batch))
-
-    # --------------------------------------------------------------------- #
-    # VAE Model Utility Functions (Training)                                #
-    # --------------------------------------------------------------------- #
-
     @final
     def encode_dists(self, x: torch.Tensor) -> Tuple[Distribution, Distribution]:
         """Get parametrisations of the latent distributions, which are sampled from during training."""
@@ -246,11 +221,6 @@ class Vae(_AeAndVaeMixin):
         z_posterior, z_prior = self.latents_handler.encoding_to_dists(z_raw)
         return z_posterior, z_prior
 
-    @final
-    def decode_partial(self, z: torch.Tensor) -> torch.Tensor:
-        """Decode latent vector z into partial reconstructions that exclude the final activation if there is one."""
-        return self._model.decode(z)
-
 
 # ========================================================================= #
 # END                                                                       #
