@@ -32,9 +32,9 @@ from tqdm import tqdm
 
 from disent.dataset import DisentDataset
 from disent.metrics import utils
-from disent.metrics.utils import make_metric
 from disent.util import to_numpy
 
+
 log = logging.getLogger(__name__)
 
 
@@ -43,17 +43,14 @@ log = logging.getLogger(__name__)
 # ========================================================================= #
 
 
-@make_metric(
-    "factor_vae", fast_kwargs=dict(num_train=700, num_eval=350, num_variance_estimate=1000)
-)  # may not be accurate, but it just takes waay too long otherwise 20+ seconds
 def metric_factor_vae(
-    dataset: DisentDataset,
-    representation_function: callable,
-    batch_size: int = 64,
-    num_train: int = 10000,
-    num_eval: int = 5000,
-    num_variance_estimate: int = 10000,
-    show_progress=False,
+        ground_truth_dataset: DisentDataset,
+        representation_function: callable,
+        batch_size: int = 64,
+        num_train: int = 10000,
+        num_eval: int = 5000,
+        num_variance_estimate: int = 10000,
+        show_progress=False,
 ):
     """
     Computes the FactorVAE disentanglement metric.
@@ -87,7 +84,7 @@ def metric_factor_vae(
     Most importantly, it circumvents the failure mode of the earlier metric, since the classifier needs to see the lowest variance in a latent dimension for a given factor to classify it correctly
 
     Args:
-      dataset: DisentDataset to be sampled from.
+      ground_truth_dataset: GroundTruthData to be sampled from.
       representation_function: Function that takes observations as input and
         outputs a dim_representation sized representation for each observation.
       batch_size: Number of points to be used to compute the training_sample.
@@ -102,68 +99,59 @@ def metric_factor_vae(
     """
 
     log.debug("Computing global variances to standardise.")
-    global_variances = _compute_variances(dataset, representation_function, num_variance_estimate)
+    global_variances = _compute_variances(ground_truth_dataset, representation_function, num_variance_estimate)
     active_dims = _prune_dims(global_variances)
 
     if not active_dims.any():
-        return {"factor_vae.train_accuracy": 0.0, "factor_vae.eval_accuracy": 0.0, "factor_vae.num_active_dims": 0}
+        return {
+            "factor_vae.train_accuracy": 0.,
+            "factor_vae.eval_accuracy": 0.,
+            "factor_vae.num_active_dims": 0
+        }
 
     log.debug("Generating training set.")
-    training_votes = _generate_training_batch(
-        dataset,
-        representation_function,
-        batch_size,
-        num_train,
-        global_variances,
-        active_dims,
-        show_progress=show_progress,
-    )
+    training_votes = _generate_training_batch(ground_truth_dataset, representation_function, batch_size, num_train, global_variances, active_dims, show_progress=show_progress)
     classifier = np.argmax(training_votes, axis=0)
     other_index = np.arange(training_votes.shape[1])
 
     # Evaluate training set accuracy
-    train_accuracy = np.sum(training_votes[classifier, other_index]) * 1.0 / np.sum(training_votes)
+    train_accuracy = np.sum(training_votes[classifier, other_index]) * 1. / np.sum(training_votes)
 
     log.debug("Generating evaluation set.")
-    eval_votes = _generate_training_batch(
-        dataset,
-        representation_function,
-        batch_size,
-        num_eval,
-        global_variances,
-        active_dims,
-        show_progress=show_progress,
-    )
+    eval_votes = _generate_training_batch(ground_truth_dataset, representation_function, batch_size, num_eval, global_variances, active_dims, show_progress=show_progress)
 
     # Evaluate evaluation set accuracy
-    eval_accuracy = np.sum(eval_votes[classifier, other_index]) * 1.0 / np.sum(eval_votes)
+    eval_accuracy = np.sum(eval_votes[classifier, other_index]) * 1. / np.sum(eval_votes)
 
     return {
-        "factor_vae.train_accuracy": train_accuracy,  # "z-min variance" -- Measuring Disentanglement: A Review of Metrics
-        "factor_vae.eval_accuracy": eval_accuracy,  # "z-min variance" -- Measuring Disentanglement: A Review of Metrics
+        "factor_vae.train_accuracy": train_accuracy,
+        "factor_vae.eval_accuracy": eval_accuracy,
         "factor_vae.num_active_dims": len(active_dims),
     }
 
 
-def _prune_dims(variances, threshold=0.0):
+def _prune_dims(variances, threshold=0.):
     """Mask for dimensions collapsed to the prior."""
     scale_z = np.sqrt(variances)
     return scale_z >= threshold
 
 
 def _compute_variances(
-    dataset: DisentDataset, representation_function: callable, batch_size: int, eval_batch_size: int = 64
+        ground_truth_dataset: DisentDataset,
+        representation_function: callable,
+        batch_size: int,
+        eval_batch_size: int = 64
 ):
     """Computes the variance for each dimension of the representation.
     Args:
-      dataset: DisentDataset to be sampled from.
+      ground_truth_dataset: GroundTruthData to be sampled from.
       representation_function: Function that takes observation as input and outputs a representation.
       batch_size: Number of points to be used to compute the variances.
       eval_batch_size: Batch size used to eval representation.
     Returns:
       Vector with the variance of each dimension.
     """
-    observations = dataset.dataset_sample_batch(batch_size, mode="input")
+    observations = ground_truth_dataset.dataset_sample_batch(batch_size, mode='input')
     representations = to_numpy(utils.obtain_representation(observations, representation_function, eval_batch_size))
     representations = np.transpose(representations)
     assert representations.shape[0] == batch_size
@@ -171,15 +159,15 @@ def _compute_variances(
 
 
 def _generate_training_sample(
-    dataset: DisentDataset,
-    representation_function: callable,
-    batch_size: int,
-    global_variances: np.ndarray,
-    active_dims: list,
+        ground_truth_dataset: DisentDataset,
+        representation_function: callable,
+        batch_size: int,
+        global_variances: np.ndarray,
+        active_dims: list,
 ) -> (int, int):
     """Sample a single training sample based on a mini-batch of ground-truth data.
     Args:
-      dataset: DisentDataset to be sampled from.
+      ground_truth_dataset: GroundTruthData to be sampled from.
       representation_function: Function that takes observation as input and
         outputs a representation.
       batch_size: Number of points to be used to compute the training_sample.
@@ -190,13 +178,13 @@ def _generate_training_sample(
       argmin: Index of representation coordinate with the least variance.
     """
     # Select random coordinate to keep fixed.
-    factor_index = np.random.randint(dataset.gt_data.num_factors)
+    factor_index = np.random.randint(ground_truth_dataset.ground_truth_data.num_factors)
     # Sample two mini batches of latent variables.
-    factors = dataset.gt_data.sample_factors(batch_size)
+    factors = ground_truth_dataset.ground_truth_data.sample_factors(batch_size)
     # Fix the selected factor across mini-batch.
     factors[:, factor_index] = factors[0, factor_index]
     # Obtain the observations.
-    observations = dataset.dataset_batch_from_factors(factors, mode="input")
+    observations = ground_truth_dataset.dataset_batch_from_factors(factors, mode='input')
     representations = to_numpy(representation_function(observations))
     local_variances = np.var(representations, axis=0, ddof=1)
     argmin = np.argmin(local_variances[active_dims] / global_variances[active_dims])
@@ -204,17 +192,17 @@ def _generate_training_sample(
 
 
 def _generate_training_batch(
-    dataset: DisentDataset,
-    representation_function: callable,
-    batch_size: int,
-    num_points: int,
-    global_variances: np.ndarray,
-    active_dims: list,
-    show_progress=False,
+        ground_truth_dataset: DisentDataset,
+        representation_function: callable,
+        batch_size: int,
+        num_points: int,
+        global_variances: np.ndarray,
+        active_dims: list,
+        show_progress=False,
 ):
     """Sample a set of training samples based on a batch of ground-truth data.
     Args:
-      dataset: DisentDataset to be sampled from.
+      ground_truth_dataset: GroundTruthData to be sampled from.
       representation_function: Function that takes observations as input and outputs a dim_representation sized representation for each observation.
       batch_size: Number of points to be used to compute the training_sample.
       num_points: Number of points to be sampled for training set.
@@ -223,15 +211,12 @@ def _generate_training_batch(
     Returns:
       (num_factors, dim_representation)-sized numpy array with votes.
     """
-    votes = np.zeros((dataset.gt_data.num_factors, global_variances.shape[0]), dtype=np.int64)
+    votes = np.zeros((ground_truth_dataset.ground_truth_data.num_factors, global_variances.shape[0]), dtype=np.int64)
     for _ in tqdm(range(num_points), disable=(not show_progress)):
-        factor_index, argmin = _generate_training_sample(
-            dataset, representation_function, batch_size, global_variances, active_dims
-        )
+        factor_index, argmin = _generate_training_sample(ground_truth_dataset, representation_function, batch_size, global_variances, active_dims)
         votes[factor_index, argmin] += 1
     return votes
 
-
 # ========================================================================= #
 # END                                                                       #
 # ========================================================================= #
